@@ -3,10 +3,12 @@ package deb
 
 import (
 	"archive/tar"
+	"bufio"
 	"bytes"
 	"compress/gzip"
 	"crypto/md5" // nolint:gas
 	"crypto/sha1"
+	"encoding/hex"
 	"errors"
 	"fmt"
 	"io"
@@ -21,6 +23,8 @@ import (
 	"github.com/goreleaser/nfpm/v2"
 	"github.com/goreleaser/nfpm/v2/deprecation"
 	"github.com/goreleaser/nfpm/v2/files"
+	"github.com/goreleaser/nfpm/v2/internal/maps"
+	"github.com/goreleaser/nfpm/v2/internal/modtime"
 	"github.com/goreleaser/nfpm/v2/internal/sign"
 	"github.com/klauspost/compress/zstd"
 	"github.com/ulikunitz/xz"
@@ -124,25 +128,27 @@ func (d *Deb) Package(info *nfpm.Info, deb io.Writer) (err error) { // nolint: f
 		return fmt.Errorf("cannot write ar header to deb file: %w", err)
 	}
 
-	if err := addArFile(w, "debian-binary", debianBinary); err != nil {
+	mtime := modtime.Get(info.MTime)
+
+	if err := addArFile(w, "debian-binary", debianBinary, mtime); err != nil {
 		return fmt.Errorf("cannot pack debian-binary: %w", err)
 	}
 
-	if err := addArFile(w, "control.tar.gz", controlTarGz); err != nil {
+	if err := addArFile(w, "control.tar.gz", controlTarGz, mtime); err != nil {
 		return fmt.Errorf("cannot add control.tar.gz to deb: %w", err)
 	}
 
-	if err := addArFile(w, dataTarballName, dataTarball); err != nil {
+	if err := addArFile(w, dataTarballName, dataTarball, mtime); err != nil {
 		return fmt.Errorf("cannot add data.tar.gz to deb: %w", err)
 	}
 
-	if info.Deb.Signature.KeyFile != "" {
+	if info.Deb.Signature.KeyFile != "" || info.Deb.Signature.SignFn != nil {
 		sig, sigType, err := doSign(info, debianBinary, controlTarGz, dataTarball)
 		if err != nil {
 			return err
 		}
 
-		if err := addArFile(w, "_gpg"+sigType, sig); err != nil {
+		if err := addArFile(w, "_gpg"+sigType, sig, mtime); err != nil {
 			return &nfpm.ErrSigningFailure{
 				Err: fmt.Errorf("add signature to ar file: %w", err),
 			}
@@ -172,7 +178,12 @@ func dpkgSign(info *nfpm.Info, debianBinary, controlTarGz, dataTarball []byte) (
 		return nil, sigType, &nfpm.ErrSigningFailure{Err: err}
 	}
 
-	sig, err := sign.PGPClearSignWithKeyID(data, info.Deb.Signature.KeyFile, info.Deb.Signature.KeyPassphrase, info.Deb.Signature.KeyID)
+	var sig []byte
+	if signFn := info.Deb.Signature.SignFn; signFn != nil {
+		sig, err = signFn(data)
+	} else {
+		sig, err = sign.PGPClearSignWithKeyID(data, info.Deb.Signature.KeyFile, info.Deb.Signature.KeyPassphrase, info.Deb.Signature.KeyID)
+	}
 	if err != nil {
 		return nil, sigType, &nfpm.ErrSigningFailure{Err: err}
 	}
@@ -193,7 +204,13 @@ func debSign(info *nfpm.Info, debianBinary, controlTarGz, dataTarball []byte) ([
 		}
 	}
 
-	sig, err := sign.PGPArmoredDetachSignWithKeyID(data, info.Deb.Signature.KeyFile, info.Deb.Signature.KeyPassphrase, info.Deb.Signature.KeyID)
+	var sig []byte
+	var err error
+	if signFn := info.Deb.Signature.SignFn; signFn != nil {
+		sig, err = signFn(data)
+	} else {
+		sig, err = sign.PGPArmoredDetachSignWithKeyID(data, info.Deb.Signature.KeyFile, info.Deb.Signature.KeyPassphrase, info.Deb.Signature.KeyID)
+	}
 	if err != nil {
 		return nil, sigType, &nfpm.ErrSigningFailure{Err: err}
 	}
@@ -214,7 +231,9 @@ Signer: {{ .Signer }}
 Date: {{ .Date }}
 Role: {{ .Role }}
 Files:
-{{range .Files}}{{ .Md5Sum }} {{ .Sha1Sum }} {{ .Size }} {{ .Name }}{{end}}
+{{range .Files -}}
+{{"\t"}}{{ hex .Md5Sum }} {{ hex .Sha1Sum }} {{ .Size }} {{ .Name }}
+{{end -}}
 `
 
 type dpkgSigData struct {
@@ -225,17 +244,18 @@ type dpkgSigData struct {
 	Info   *nfpm.Info
 }
 type dpkgSigFileLine struct {
-	Md5Sum  [16]byte
-	Sha1Sum [20]byte
+	Md5Sum  []byte
+	Sha1Sum []byte
 	Size    int
 	Name    string
 }
 
 func newDpkgSigFileLine(name string, fileContent []byte) dpkgSigFileLine {
+	md5Sum, sha1Sum := md5.Sum(fileContent), sha1.Sum(fileContent)
 	return dpkgSigFileLine{
 		Name:    name,
-		Md5Sum:  md5.Sum(fileContent),
-		Sha1Sum: sha1.Sum(fileContent),
+		Md5Sum:  md5Sum[:],
+		Sha1Sum: sha1Sum[:],
 		Size:    len(fileContent),
 	}
 }
@@ -243,7 +263,7 @@ func newDpkgSigFileLine(name string, fileContent []byte) dpkgSigFileLine {
 func readDpkgSigData(info *nfpm.Info, debianBinary, controlTarGz, dataTarball []byte) (io.Reader, error) {
 	data := dpkgSigData{
 		Signer: info.Deb.Signature.Signer,
-		Date:   time.Now(),
+		Date:   modtime.Get(info.MTime),
 		Role:   info.Deb.Signature.Type,
 		Files: []dpkgSigFileLine{
 			newDpkgSigFileLine("debian-binary", debianBinary),
@@ -251,7 +271,9 @@ func readDpkgSigData(info *nfpm.Info, debianBinary, controlTarGz, dataTarball []
 			newDpkgSigFileLine("data.tar.gz", dataTarball),
 		},
 	}
-	temp, _ := template.New("dpkg-sig").Parse(dpkgSigTemplate)
+	temp, _ := template.New("dpkg-sig").Funcs(template.FuncMap{
+		"hex": hex.EncodeToString,
+	}).Parse(dpkgSigTemplate)
 	buf := &bytes.Buffer{}
 	err := temp.Execute(buf, data)
 	if err != nil {
@@ -278,12 +300,12 @@ func (*Deb) SetPackagerDefaults(info *nfpm.Info) {
 	}
 }
 
-func addArFile(w *ar.Writer, name string, body []byte) error {
+func addArFile(w *ar.Writer, name string, body []byte, date time.Time) error {
 	header := ar.Header{
 		Name:    files.ToNixPath(name),
 		Size:    int64(len(body)),
 		Mode:    0o644,
-		ModTime: time.Now(),
+		ModTime: date,
 	}
 	if err := w.WriteHeader(&header); err != nil {
 		return fmt.Errorf("cannot write file header: %w", err)
@@ -379,14 +401,14 @@ func createFilesInsideDataTar(info *nfpm.Info, tw *tar.Writer) (md5buf bytes.Buf
 				Format:   tar.FormatGNU,
 				Uname:    file.FileInfo.Owner,
 				Gname:    file.FileInfo.Group,
-				ModTime:  file.FileInfo.MTime,
+				ModTime:  modtime.Get(info.MTime),
 			})
 		case files.TypeSymlink:
 			err = newItemInsideTar(tw, []byte{}, &tar.Header{
 				Name:     files.AsExplicitRelativePath(file.Destination),
 				Linkname: file.Source,
 				Typeflag: tar.TypeSymlink,
-				ModTime:  file.FileInfo.MTime,
+				ModTime:  modtime.Get(info.MTime),
 				Format:   tar.FormatGNU,
 			})
 		case files.TypeDebChangelog:
@@ -498,7 +520,7 @@ func createChangelogInsideDataTar(
 		return 0, err
 	}
 
-	if err = newFileInsideTar(tarw, fileName, changelogData); err != nil {
+	if err = newFileInsideTar(tarw, fileName, changelogData, modtime.Get(info.MTime)); err != nil {
 		return 0, err
 	}
 
@@ -542,19 +564,19 @@ func createControl(instSize int64, md5sums []byte, info *nfpm.Info) (controlTarG
 		return nil, err
 	}
 
-	filesToCreate := map[string][]byte{
-		"./control":   body.Bytes(),
-		"./md5sums":   md5sums,
-		"./conffiles": conffiles(info),
+	mtime := modtime.Get(info.MTime)
+	if err := newFileInsideTar(out, "./control", body.Bytes(), mtime); err != nil {
+		return nil, err
+	}
+	if err := newFileInsideTar(out, "./md5sums", md5sums, mtime); err != nil {
+		return nil, err
+	}
+	if err := newFileInsideTar(out, "./conffiles", conffiles(info), mtime); err != nil {
+		return nil, err
 	}
 
-	triggers := createTriggers(info)
-	if len(triggers) > 0 {
-		filesToCreate["./triggers"] = triggers
-	}
-
-	for name, content := range filesToCreate {
-		if err := newFileInsideTar(out, name, content); err != nil {
+	if triggers := createTriggers(info); len(triggers) > 0 {
+		if err := newFileInsideTar(out, "./triggers", triggers, mtime); err != nil {
 			return nil, err
 		}
 	}
@@ -564,41 +586,44 @@ func createControl(instSize int64, md5sums []byte, info *nfpm.Info) (controlTarG
 		mode     int64
 	}
 
-	specialFiles := map[string]*fileAndMode{}
-	specialFiles[info.Scripts.PreInstall] = &fileAndMode{
-		fileName: "preinst",
-		mode:     0o755,
-	}
-	specialFiles[info.Scripts.PostInstall] = &fileAndMode{
-		fileName: "postinst",
-		mode:     0o755,
-	}
-	specialFiles[info.Scripts.PreRemove] = &fileAndMode{
-		fileName: "prerm",
-		mode:     0o755,
-	}
-	specialFiles[info.Scripts.PostRemove] = &fileAndMode{
-		fileName: "postrm",
-		mode:     0o755,
-	}
-	specialFiles[info.Overridables.Deb.Scripts.Rules] = &fileAndMode{
-		fileName: "rules",
-		mode:     0o755,
-	}
-	specialFiles[info.Overridables.Deb.Scripts.Templates] = &fileAndMode{
-		fileName: "templates",
-		mode:     0o644,
-	}
-	specialFiles[info.Overridables.Deb.Scripts.Config] = &fileAndMode{
-		fileName: "config",
-		mode:     0o755,
+	specialFiles := map[string]*fileAndMode{
+		"preinst": {
+			fileName: info.Scripts.PreInstall,
+			mode:     0o755,
+		},
+		"postinst": {
+			fileName: info.Scripts.PostInstall,
+			mode:     0o755,
+		},
+		"prerm": {
+			fileName: info.Scripts.PreRemove,
+			mode:     0o755,
+		},
+		"postrm": {
+			fileName: info.Scripts.PostRemove,
+			mode:     0o755,
+		},
+		"rules": {
+			fileName: info.Overridables.Deb.Scripts.Rules,
+			mode:     0o755,
+		},
+		"templates": {
+			fileName: info.Overridables.Deb.Scripts.Templates,
+			mode:     0o644,
+		},
+		"config": {
+			fileName: info.Overridables.Deb.Scripts.Config,
+			mode:     0o755,
+		},
 	}
 
-	for path, destMode := range specialFiles {
-		if path != "" {
-			if err := newFilePathInsideTar(out, path, destMode.fileName, destMode.mode); err != nil {
-				return nil, err
-			}
+	for _, filename := range maps.Keys(specialFiles) {
+		dets := specialFiles[filename]
+		if dets.fileName == "" {
+			continue
+		}
+		if err := newFilePathInsideTar(out, dets.fileName, filename, dets.mode, mtime); err != nil {
+			return nil, err
 		}
 	}
 
@@ -621,18 +646,18 @@ func newItemInsideTar(out *tar.Writer, content []byte, header *tar.Header) error
 	return nil
 }
 
-func newFileInsideTar(out *tar.Writer, name string, content []byte) error {
+func newFileInsideTar(out *tar.Writer, name string, content []byte, modtime time.Time) error {
 	return newItemInsideTar(out, content, &tar.Header{
 		Name:     files.AsExplicitRelativePath(name),
 		Size:     int64(len(content)),
 		Mode:     0o644,
-		ModTime:  time.Now(),
+		ModTime:  modtime,
 		Typeflag: tar.TypeReg,
 		Format:   tar.FormatGNU,
 	})
 }
 
-func newFilePathInsideTar(out *tar.Writer, path, dest string, mode int64) error {
+func newFilePathInsideTar(out *tar.Writer, path, dest string, mode int64, modtime time.Time) error {
 	file, err := os.Open(path) //nolint:gosec
 	if err != nil {
 		return err
@@ -645,7 +670,7 @@ func newFilePathInsideTar(out *tar.Writer, path, dest string, mode int64) error
 		Name:     files.AsExplicitRelativePath(dest),
 		Size:     int64(len(content)),
 		Mode:     mode,
-		ModTime:  time.Now(),
+		ModTime:  modtime,
 		Typeflag: tar.TypeReg,
 		Format:   tar.FormatGNU,
 	})
@@ -709,6 +734,9 @@ Replaces: {{join .}}
 {{- with nonEmpty .Info.Provides}}
 Provides: {{join .}}
 {{- end }}
+{{- with .Info.Deb.Predepends}}
+Pre-Depends: {{join .}}
+{{- end }}
 {{- with .Info.Depends}}
 Depends: {{join .}}
 {{- end }}
@@ -748,8 +776,20 @@ func writeControl(w io.Writer, data controlData) error {
 			return strings.Trim(strings.Join(strs, ", "), " ")
 		},
 		"multiline": func(strs string) string {
-			ret := strings.ReplaceAll(strs, "\n", "\n ")
-			return strings.Trim(ret, " \n")
+			var b strings.Builder
+			s := bufio.NewScanner(strings.NewReader(strings.TrimSpace(strs)))
+			s.Scan()
+			b.Write(bytes.TrimSpace(s.Bytes()))
+			for s.Scan() {
+				b.WriteString("\n ")
+				l := bytes.TrimSpace(s.Bytes())
+				if len(l) == 0 {
+					b.WriteByte('.')
+				} else {
+					b.Write(l)
+				}
+			}
+			return b.String()
 		},
 		"nonEmpty": func(strs []string) []string {
 			var result []string
