@@ -5,6 +5,7 @@ import (
 	"bytes"
 	"compress/gzip"
 	"crypto/md5" // nolint: gosec
+	"crypto/sha1"
 	"encoding/hex"
 	"errors"
 	"flag"
@@ -13,9 +14,11 @@ import (
 	"os"
 	"path"
 	"path/filepath"
+	"slices"
 	"strconv"
 	"strings"
 	"testing"
+	"time"
 
 	"github.com/blakesmith/ar"
 	"github.com/goreleaser/chglog"
@@ -30,6 +33,8 @@ import (
 // nolint: gochecknoglobals
 var update = flag.Bool("update", false, "update .golden files")
 
+var mtime = time.Date(2023, 11, 5, 23, 15, 17, 0, time.UTC)
+
 func exampleInfo() *nfpm.Info {
 	return nfpm.WithDefaults(&nfpm.Info{
 		Name:        "foo",
@@ -88,6 +93,9 @@ func exampleInfo() *nfpm.Info {
 					Type:        files.TypeDir,
 				},
 			},
+			Deb: nfpm.Deb{
+				Predepends: []string{"less"},
+			},
 		},
 	})
 }
@@ -266,8 +274,8 @@ func TestSpecialFiles(t *testing.T) {
 	var w bytes.Buffer
 	out := tar.NewWriter(&w)
 	filePath := "testdata/templates.golden"
-	require.Error(t, newFilePathInsideTar(out, "doesnotexit", "templates", 0o644))
-	require.NoError(t, newFilePathInsideTar(out, filePath, "templates", 0o644))
+	require.Error(t, newFilePathInsideTar(out, "doesnotexit", "templates", 0o644, mtime))
+	require.NoError(t, newFilePathInsideTar(out, filePath, "templates", 0o644, mtime))
 	in := tar.NewReader(&w)
 	header, err := in.Next()
 	require.NoError(t, err)
@@ -520,7 +528,7 @@ func TestMultilineFields(t *testing.T) {
 		Info: nfpm.WithDefaults(&nfpm.Info{
 			Name:        "multiline",
 			Arch:        "riscv64",
-			Description: "This field is a\nmultiline field\nthat should work.",
+			Description: "This field is a\nmultiline field\n\nthat should work.",
 			Priority:    "extra",
 			Version:     "1.0.0",
 			Section:     "default",
@@ -780,13 +788,13 @@ func TestMD5Sums(t *testing.T) {
 	md5sumsFile := extractFileFromTar(t, inflate(t, "gz", controlTarGz), "./md5sums")
 
 	lines := strings.Split(strings.TrimRight(string(md5sumsFile), "\n"), "\n")
-	require.Equal(t, nFiles, len(lines), string(md5sumsFile))
+	require.Len(t, lines, nFiles, string(md5sumsFile))
 
 	dataTar := inflate(t, tarballName, dataTarball)
 
 	for _, line := range lines {
 		parts := strings.Fields(line)
-		require.Equal(t, len(parts), 2)
+		require.Len(t, parts, 2)
 
 		md5sum, fileName := parts[0], parts[1]
 
@@ -852,8 +860,8 @@ func TestDirectories(t *testing.T) {
 	require.Equal(t, h.Typeflag, byte(tar.TypeDir))
 	h = extractFileHeaderFromTar(t, dataTarball, "/etc/bar")
 	require.Equal(t, h.Typeflag, byte(tar.TypeDir))
-	require.Equal(t, h.Mode, int64(0o700))
-	require.Equal(t, h.Uname, "test")
+	require.Equal(t, int64(0o700), h.Mode)
+	require.Equal(t, "test", h.Uname)
 	h = extractFileHeaderFromTar(t, dataTarball, "/etc/baz")
 	require.Equal(t, h.Typeflag, byte(tar.TypeDir))
 
@@ -957,7 +965,29 @@ func TestDebsigsSignatureError(t *testing.T) {
 	require.Error(t, err)
 
 	var expectedError *nfpm.ErrSigningFailure
-	require.True(t, errors.As(err, &expectedError))
+	require.ErrorAs(t, err, &expectedError)
+}
+
+func TestDebsigsSignatureCallback(t *testing.T) {
+	info := exampleInfo()
+	info.Deb.Signature.SignFn = func(r io.Reader) ([]byte, error) {
+		return sign.PGPArmoredDetachSignWithKeyID(r, "../internal/sign/testdata/privkey.asc", "hunter2", nil)
+	}
+
+	var deb bytes.Buffer
+	err := Default.Package(info, &deb)
+	require.NoError(t, err)
+
+	debBinary := extractFileFromAr(t, deb.Bytes(), "debian-binary")
+	controlTarGz := extractFileFromAr(t, deb.Bytes(), "control.tar.gz")
+	dataTarball := extractFileFromAr(t, deb.Bytes(), findDataTarball(t, deb.Bytes()))
+	signature := extractFileFromAr(t, deb.Bytes(), "_gpgorigin")
+
+	message := io.MultiReader(bytes.NewReader(debBinary),
+		bytes.NewReader(controlTarGz), bytes.NewReader(dataTarball))
+
+	err = sign.PGPVerify(message, signature, "../internal/sign/testdata/pubkey.asc")
+	require.NoError(t, err)
 }
 
 func TestDpkgSigSignature(t *testing.T) {
@@ -973,8 +1003,10 @@ func TestDpkgSigSignature(t *testing.T) {
 
 	signature := extractFileFromAr(t, deb.Bytes(), "_gpgbuilder")
 
-	err = sign.PGPReadMessage(signature, "../internal/sign/testdata/pubkey.asc")
+	msg, err := sign.PGPReadMessage(signature, "../internal/sign/testdata/pubkey.asc")
 	require.NoError(t, err)
+
+	require.NoError(t, verifyDpkgSigFileHashes(extractAllFilesFromAr(t, deb.Bytes()), string(msg)))
 }
 
 func TestDpkgSigSignatureError(t *testing.T) {
@@ -987,7 +1019,27 @@ func TestDpkgSigSignatureError(t *testing.T) {
 	require.Error(t, err)
 
 	var expectedError *nfpm.ErrSigningFailure
-	require.True(t, errors.As(err, &expectedError))
+	require.ErrorAs(t, err, &expectedError)
+}
+
+func TestDpkgSigSignatureCallback(t *testing.T) {
+	info := exampleInfo()
+	info.Deb.Signature.SignFn = func(r io.Reader) ([]byte, error) {
+		return sign.PGPClearSignWithKeyID(r, "../internal/sign/testdata/privkey.asc", "hunter2", nil)
+	}
+	info.Deb.Signature.Method = "dpkg-sig"
+	info.Deb.Signature.Signer = "bob McRobert"
+
+	var deb bytes.Buffer
+	err := Default.Package(info, &deb)
+	require.NoError(t, err)
+
+	signature := extractFileFromAr(t, deb.Bytes(), "_gpgbuilder")
+
+	msg, err := sign.PGPReadMessage(signature, "../internal/sign/testdata/pubkey.asc")
+	require.NoError(t, err)
+
+	require.NoError(t, verifyDpkgSigFileHashes(extractAllFilesFromAr(t, deb.Bytes()), string(msg)))
 }
 
 func TestDisableGlobbing(t *testing.T) {
@@ -1137,7 +1189,7 @@ func TestIgnoreUnrelatedFiles(t *testing.T) {
 	require.NoError(t, err)
 
 	contents := tarContents(t, inflate(t, tarballName, dataTarball))
-	require.Len(t, contents, 0)
+	require.Empty(t, contents)
 }
 
 func extractFileFromTar(tb testing.TB, tarFile []byte, filename string) []byte {
@@ -1354,6 +1406,26 @@ func extractFileFromAr(tb testing.TB, arFile []byte, filename string) []byte {
 	return nil
 }
 
+func extractAllFilesFromAr(tb testing.TB, arFile []byte) map[string][]byte {
+	tb.Helper()
+
+	tr := ar.NewReader(bytes.NewReader(arFile))
+	files := make(map[string][]byte)
+	for {
+		hdr, err := tr.Next()
+		if errors.Is(err, io.EOF) {
+			break // End of archive
+		}
+		require.NoError(tb, err)
+
+		fileContents, err := io.ReadAll(tr)
+		require.NoError(tb, err)
+
+		files[hdr.Name] = fileContents
+	}
+	return files
+}
+
 func TestEmptyButRequiredDebFields(t *testing.T) {
 	item := nfpm.WithDefaults(&nfpm.Info{
 		Name:    "foo",
@@ -1460,3 +1532,45 @@ func (zrc *zstdReadCloser) Close() error {
 
 	return nil
 }
+
+func verifyDpkgSigFileHashes(arFiles map[string][]byte, msg string) error {
+	_, hashes, ok := strings.Cut(msg, "Files:")
+	if !ok {
+		return errors.New("expected Files section in dpkg-sig message")
+	}
+	lines := strings.Split(hashes, "\n")
+	for i := range lines {
+		lines[i] = strings.TrimSpace(lines[i])
+		if lines[i] == "" {
+			continue
+		}
+		var md5Hex, sha1Hex, size, name string
+		if n, err := fmt.Sscanln(lines[i], &md5Hex, &sha1Hex, &size, &name); err != nil {
+			return err
+		} else if n != 4 {
+			return fmt.Errorf("expected 4 elements in line %q, but got %d", lines[i], n)
+		}
+
+		md5Sum, err := hex.DecodeString(md5Hex)
+		if err != nil {
+			return err
+		}
+		sha1Sum, err := hex.DecodeString(sha1Hex)
+		if err != nil {
+			return err
+		}
+
+		content, ok := arFiles[name]
+		if !ok {
+			return fmt.Errorf("dpkg-sig message contains hash of file %q, but the package does not contain the file", name)
+		}
+		actualMD5Sum, actualSHA1Sum := md5.Sum(content), sha1.Sum(content)
+		if !slices.Equal(actualMD5Sum[:], md5Sum) {
+			return fmt.Errorf("file %q has invalid MD5 sum", name)
+		}
+		if !slices.Equal(actualSHA1Sum[:], sha1Sum) {
+			return fmt.Errorf("file %q has invalid SHA1 sum", name)
+		}
+	}
+	return nil
+}
